/*  plumb - the pipe shell - key-value lists
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <assert.h>
#include "kv.h"

kv_t *kv(char *key, char *value)
{
	kv_t *k;

	k = malloc(sizeof(kv_t));
	k->key   = key;
	k->value = value;
	k->next  = NULL;
	return k;
}


/* append item to the list; if list is NULL, allocate a new list */
kvlist_t *kvlist_append(kvlist_t *list, kv_t *item)
{
	assert(item != NULL);

	if (list == NULL) {
		list = malloc(sizeof(kvlist_t));
		list->head = list->tail = NULL;
		list->len = 0;
	}

	/* separate if, just in case the caller passes an empty list */
	if (list->tail != NULL)
		list->tail->next = item;
	else
		list->head = item;

	list->tail = item;
	item->next = NULL;
	list->len++;
	return list;
}

/* static because it does not unlink */
static void kv_free(kv_t *k)
{
	if (k->key != NULL)
		free(k->key);
	if (k->value != NULL)
		free(k->value);
	free(k);
}

void kvlist_free(kvlist_t *list)
{
	kv_t *i, *next;
	for(i = list->head; i != NULL; i = next) {
		next = i->next;
		kv_free(i);
	}
	free(list);
}
